% pop_InterpMap() - creates interpolation map
%
% Usage: pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,MoveValue,Run)
%  
% Inputs:
%     EEG            - EEGLAB datasetindices of channels to interpolate.
%     ChanListNum2   - Index of Channel cluster of interest 
%     ChanListLab2   - Labels of Channel cluster of interest 
%     currentPos     - Current center channels
%     MoveValue      - Move to next to perivous channels
%     Run            - Switch for which process to run
%
% Author: Matthew Phillip Gunn, SIUC-INL, 2022%
% Copyright (C) 2022, Matthew Phillip Gunn, SIUC-INL, matthewpgunn@gmail.com
%
% This file is part of EEGLAB, see http://www.eeglab.org for the
% documentation and details.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
% 1. Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer.
%
% 2. Redistributions in binary form must reproduce the above copyright
% notice, this list of conditions and the following disclaimer in the
% documentation and/or other materials provided with the distribution.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
function [EEG] = pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,MoveValue,Run)

%     'AA = get(gca,''Parent'');AB = get(AA,''userdata'');ChanListNum2 = AB.ImapchansNL;ChanListLab2 = AB.chanlocsNL;pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,1,1);'
%            { 'Style', 'pushbutton', 'string', '>',  'callback', [ 'pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,1,1);'  ] } ...
if Run == 0
    Ti = sprintf('Channel Interpolation Maps');
    ButtonSection = {...
        { 'Style', 'text', 'string', ''} ...
        { 'Style', 'text', 'string', ''} ...
        { 'Style', 'pushbutton', 'string', '<',  'callback', ['AA = get(gca,''Parent'');AB = get(AA,''userdata'');ChanListNum2 = AB.ImapchansNL;ChanListLab2 = AB.chanlocsNL;pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,-1,1);' ] } ...
        { 'Style', 'edit'      , 'string', num2str(currentPos),'tag','CurChanEditNum','Value',currentPos, 'callback', ['currentPos = get(findobj(AA, ''tag'', ''CurChanEditNum''),''Value''); pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,0,2);'] } ...
        { 'Style', 'pushbutton', 'string', '>',  'callback', [    'AA = get(gca,''Parent'');AB = get(AA,''userdata'');ChanListNum2 = AB.ImapchansNL;ChanListLab2 = AB.chanlocsNL;pop_InterpMap(EEG,ChanListNum2,ChanListLab2,currentPos,1,1);'] } ...
        { 'Style', 'text', 'string', ''} ...
        { 'Style', 'text', 'string', ''}, ...
        };
    geometry =  {...
        [ 1 ] ...
        [1.5  1    0.2 3 1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [1]...
        [1]...
        [2    0.35 0.5 3 1]...
        [2    0.35 0.5 3 1]...
        [1]...
        [1]...
        [1]...
        [1]...
        [1]...
        [1]...
        [1]...
        [1]...
        [2 0.7  0.7 1 0.7 0.7 .75] }; %Last line is button line
    uilist   = {....
        { 'Style', 'text', 'string', Ti, 'fontweight', 'bold'  }, ...
        { 'Style', 'text', 'string', 'Blue/Boxed Channels'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'will be used for'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'interpolation analysis'},{},{},{},{}, ...
        { }, ...
        { 'Style', 'text', 'string', 'Red/Non-boxed Channels'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'will not be used for'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'interpolation analysis'},{},{},{},{}, ...
        { }, ...
        { 'Style', 'text', 'string', 'These maps can be used'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'later and load into'},{},{},{},{}, ...
        { 'Style', 'text', 'string', 'interpolation analysis'},{},{},{},{}, ...
        { }, ...
        { }, ...
        { 'Style', 'text', 'string', 'Central Channel', 'fontweight', 'bold' },{},{},{},{}, ...
        { 'Style', 'text', 'string', EEG.chanlocs(1,1).labels, 'fontweight', 'bold','tag','CurChanLabel' },{},{},{},{}, ...
        { }, ...
        { }, ...
        { }, ...
        { }, ...
        { }, ...
        { }, ...
        { 'Style', 'text', 'string', strcat('Filename InterpMapUserBased.txt')},...
        { 'Style', 'text', 'string', strcat('File saving to',{' '}, string(pwd))},...
        ButtonSection{:},...
        };
    
    userdata.chansNL    = ChanListNum2;
    userdata.chanlocsNL = EEG.chanlocs;
    userdata.chaninfoNL = EEG.chaninfo;
    userdata.ImapchansNL = ChanListNum2;
    close();
    CC = nonzeros(ChanListNum2(1,:)');
    [results userdata returnmode] = inputguiMG(EEG,CC,'geometry', geometry, 'uilist', uilist, 'helpcom', ...
        'pophelp(''pop_chanedit'');', 'title', 'Edit channel info -- pop_chanedit()', ...
        'userdata', userdata);
    
elseif Run == 1
    AA = get(gca,'Parent');
    AB = get(AA,'userdata');
    AC = get(gca,'Children');
    AD = get(AC,'userdata');
    CL_NL = AB.chanlocsNL;
    CI_NL = AB.chaninfoNL;
    CM_NL = AB.ImapchansNL;
    AD = AD(~cellfun('isempty',AD));
    AD = cellstr(string(flip(AD)));
    Index = find(not(cellfun('isempty',(strfind(AD,':1')))));
    currentPos = get(findobj(AA, 'tag', 'CurChanEditNum'),'string');
    
    if isnan(str2double(currentPos))
        ALcmp = {EEG.chanlocs(1:end).labels}';
        currentPos =  find(strcmpi(ALcmp, currentPos));
    else
        currentPos =str2double(currentPos);
    end
    currentPosO = currentPos;
    currentPos = currentPos + MoveValue;
    
    if currentPos > 0 && currentPos < size(CL_NL,2)+1
        set(findobj(AA, 'tag', 'CurChanEditNum'),'string',currentPos)
        set(findobj(AA, 'tag', 'CurChanLabel'),'string',CL_NL(currentPos).labels)
        set(findobj(AA, 'tag', 'CurChanEditNum'),'Value',currentPos)
        index_of_Channels = nonzeros(AB.ImapchansNL(currentPos,:)');
        cla()
        topoplotMG( index_of_Channels,CL_NL,'chaninfo', CI_NL, ...
            'electrodes','labels','style', 'blank', 'emarkersize1chan', 2);
        
        userdata.chansNL    = ChanListNum2;
        userdata.chanlocsNL = EEG.chanlocs;
        userdata.chaninfoNL = EEG.chaninfo;
        NewLine = [Index',zeros(1,(size(ChanListNum2,2)-size(Index,1)-1))];
        ChanListNum2(currentPosO,2:end) = NewLine;
        userdata.ImapchansNL = ChanListNum2;
    else
        userdata.chansNL    = ChanListNum2;
        userdata.chanlocsNL = EEG.chanlocs;
        userdata.chaninfoNL = EEG.chaninfo;
        userdata.ImapchansNL = ChanListNum2;
    end
    set(get(gca,'Parent'), 'UserData', userdata);
    fileID = fopen('InterpMapUserBased.txt','w');
    PrintTab = ' %.7f \t ';
    for ii=1:size(ChanListNum2,2)-1
        PrintTab = strcat(PrintTab,' %.7f \t ');
    end
    PrintTab = strcat('\n ',PrintTab);
    for iii=1:size(EEG.chanlocs,2)
        fprintf(fileID, PrintTab,...
            ChanListNum2(iii,:));
    end
    fclose( 'all' );% or fclose(fileID);
    
    
elseif Run ==2
    AA = get(gca,'Parent');
    AB = get(AA,'userdata');
    AC = get(gca,'Children');
    AD = get(AC,'userdata');
    CL_NL = AB.chanlocsNL;
    CI_NL = AB.chaninfoNL;
    CM_NL = AB.ImapchansNL;
    AD = AD(~cellfun('isempty',AD));
    AD = cellstr(string(flip(AD)));
    Index = find(not(cellfun('isempty',(strfind(AD,':1')))));
    currentPos = get(findobj(AA, 'tag', 'CurChanEditNum'),'string');
    
    if isnan(str2double(currentPos))
        ALcmp = {EEG.chanlocs(1:end).labels}';
        currentPos =  find(strcmpi(ALcmp, currentPos));
    else
        currentPos =str2double(currentPos);
    end
    if currentPos > 0 && currentPos < size(CL_NL,2)+1
        % CurChan = AB.chansNL(currentPos,1);
        set(findobj(AA, 'tag', 'CurChanEditNum'),'string',currentPos)
        set(findobj(AA, 'tag', 'CurChanLabel'),'string',CL_NL(currentPos).labels)
        set(findobj(AA, 'tag', 'CurChanEditNum'),'Value',currentPos)
        index_of_Channels = nonzeros(AB.ImapchansNL(currentPos,:)');
        cla()
        %         drawnow()
        topoplotMG( index_of_Channels,CL_NL,'chaninfo', CI_NL, ...
            'electrodes','labels','style', 'blank', 'emarkersize1chan', 2);        
        userdata.chansNL    = ChanListNum2;
        userdata.chanlocsNL = EEG.chanlocs;
        userdata.chaninfoNL = EEG.chaninfo;
        NewLine = [Index',zeros(1,(size(ChanListNum2,2)-size(Index,1)-1))];
        ChanListNum2(currentPos,2:end) = NewLine;
        userdata.ImapchansNL = ChanListNum2;
    else
        userdata.chansNL    = ChanListNum2;
        userdata.chanlocsNL = EEG.chanlocs;
        userdata.chaninfoNL = EEG.chaninfo;
        userdata.ImapchansNL = ChanListNum2;
    end
    end
    set(get(gca,'Parent'), 'UserData', userdata);
    fileID = fopen('InterpMapUserBased.txt','w');
    PrintTab = ' %.7f \t ';
    for ii=1:size(ChanListNum2,2)-1
        PrintTab = strcat(PrintTab,' %.7f \t ');
    end
    PrintTab = strcat('\n ',PrintTab);
    for iii=1:size(EEG.chanlocs,2)
        fprintf(fileID, PrintTab,...
            ChanListNum2(iii,:));
    end
    fclose( 'all' );% or fclose(fileID);
end

%
% function [com] = pop_InterpMapI(EEG,ChanListNum2,ChanListLab2,currentPos,MoveValue,Run)
% global currentPos;
% global CurChan;
% AA = get(gca,'Parent');
% AB = get(AA,'userdata');
% AC = get(gca,'Children');
% AD = get(AC,'userdata');
%
% D = get(AA,'tag');
% CL_NL = AB.chanlocsNL;
% CI_NL = AB.chaninfoNL;
%
% if currentPos > 0 && currentPos < size(CL_NL,2)
%     currentPos = currentPos + MoveValue;
%    % CurChan = AB.chansNL(currentPos,1);
%     index_of_Channels = nonzeros(AB.chansNL(currentPos,:)');
%     cla()
%     drawnow()
%     topoplotMG( index_of_Channels,CL_NL,'chaninfo', CI_NL, ...
%         'electrodes','labels','style', 'blank', 'emarkersize1chan', 2);
% end
%
%
% function [com] = pop_InterpMap1(gca,MoveValue,currentPos)
%
% global currentPos;
% global CurChan;
% AA = get(gca,'Parent');
% AB = get(AA,'userdata');
% AC = get(gca,'Children');
% AD = get(AC,'userdata');
%
% D = get(AA,'tag');
% CL_NL = AB.chanlocsNL;
% CI_NL = AB.chaninfoNL;
%
% if currentPos > 0 && currentPos < size(CL_NL,2)
%     currentPos = currentPos + MoveValue;
%    % CurChan = AB.chansNL(currentPos,1);
%     index_of_Channels = nonzeros(AB.chansNL(currentPos,:)');
%     cla()
%     drawnow()
%     topoplotMG( index_of_Channels,CL_NL,'chaninfo', CI_NL, ...
%         'electrodes','labels','style', 'blank', 'emarkersize1chan', 2);
% end
%

